import { Router, Request, Response } from "express";

import { Response as HttpResponse } from "./http-petition";
import { HttpController } from "./http-controller";

export abstract class HttpResourceController extends HttpController {
  constructor() {
    super();
  }

  public abstract index(): HttpResponse | Promise<HttpResponse>;
  public abstract store(): HttpResponse | Promise<HttpResponse>;
  public abstract show(): HttpResponse | Promise<HttpResponse>;
  public abstract update(): HttpResponse | Promise<HttpResponse>;
  public abstract destroy(): HttpResponse | Promise<HttpResponse>;

  public generateRoutes(router: Router, path: string) {
    router.get(path, (req: Request, res: Response) =>
      this.getResponse(req, res, "index")
    );
    router.post(path, (req: Request, res: Response) =>
      this.getResponse(req, res, "store")
    );
    router.get(`${path}/:id`, (req: Request, res: Response) =>
      this.getResponse(req, res, "show")
    );
    router.put(`${path}/:id`, (req: Request, res: Response) =>
      this.getResponse(req, res, "update")
    );
    router.delete(`${path}/:id`, (req: Request, res: Response) =>
      this.getResponse(req, res, "destroy")
    );
  }

  private getResponse(req: Request, res: Response, method: string) {
    this.petition = {
      request: { ...req, payload: req.body },
      response: { ...res, status: "", headers: null, payload: {} }
    };

    switch (method) {
      case "index":
        return this.index();
      case "store":
        return this.store();
      case "show":
        return this.show();
      case "update":
        return this.update();
      case "destroy":
        return this.destroy();
    }
  }
}
