import { randomInt } from "crypto";
import Db from "../db";

class FakeModel {
  private _fakeData: any;
  private _name: string;

  constructor(fakeData?: any) {
    this._fakeData = fakeData;
  }

  public get fakeData() {
    return this._fakeData;
  }

  public get name() {
    return this._name;
  }

  public set name(name: string) {
    this._name = name;
  }

  findByPk(id: number) {
    return new FakeModel({ id, title: "Test title", body: "Test body" });
  }

  findAll() {
    return Array<FakeModel>();
  }

  build(modelData: any) {
    modelData._id = randomInt(9999);
    modelData.save = () => {
      return modelData;
    };

    modelData.get = () => {
      return modelData;
    };

    return modelData;
  }

  update(values: any, options: any) {
    return [1];
  }

  destroy(options: any) {
    return true;
  }

  get() {
    return this._fakeData;
  }
}

class FakeDbConnector {
  private static _instance: FakeDbConnector;

  _models: {
    [key: string]: FakeModel;
  };

  private constructor() {
    this._models = { fake: new FakeModel() };
  }

  public static get instance() {
    return this._instance || (this._instance = new this());
  }
}

export default class FakeDb extends Db {
  private _fakeDbConnector: FakeDbConnector;

  constructor() {
    const noInit = true;
    super(noInit);
    this.initFakeDbConnector();
  }

  private async initFakeDbConnector() {
    this._fakeDbConnector = FakeDbConnector.instance;

    const models = ["posts"];
    const fakeModel = new FakeModel();

    models.forEach(model => {
      fakeModel.name = model;
      this._fakeDbConnector._models[fakeModel.name] = fakeModel;
    });
  }

  public fakeModels() {
    return this._fakeDbConnector._models;
  }

  public add(): any {
    return null;
  }
}
