import FakeDb from "../";
import Post from "../../../entities/post";
import PostDb from "../../db/post";

export default class FakePostDb extends PostDb {
  private _fakeDb: FakeDb;

  constructor(_db: FakeDb) {
    super(_db);
    this._fakeDb = _db;
  }

  public async getById(id: number): Promise<any> {
    const result = await this._fakeDb.fakeModels().posts.findByPk(id);
    if (!result) {
      return null;
    }

    return { id, ...result.get() };
  }

  public async getAll(): Promise<Array<any>> {
    const result = await this._fakeDb.fakeModels().posts.findAll();
    if (!result) {
      return null;
    }

    return result;
  }

  public async add(post: Post): Promise<any> {
    const result = await this._fakeDb
      .fakeModels()
      .posts.build({ ...post })
      .save();

    if (!result) {
      return null;
    }

    return result.get();
  }

  public async update(postData: any): Promise<number> {
    const result = await this._fakeDb.fakeModels().posts.update(
      { ...postData },
      {
        where: {
          id: Number(postData.id)
        }
      }
    );

    if (!result[0]) {
      return null;
    }

    return result[0];
  }

  public async delete(post: Post): Promise<any> {
    const result = await this._fakeDb.fakeModels().posts.destroy({
      where: {
        id: post.id
      }
    });

    if (!result) {
      return null;
    }

    return result;
  }
}
