import { Request, Response } from "express";
import * as jwt from "jsonwebtoken";

export default (req: Request, res: Response, next: Function) => {
  const token: string = req.headers.authorization;
  if (!token) {
    return res.status(403).json({
      auth: false,
      status: "NO_OK",
      code: 407,
      message: "No token provided."
    });
  }

  try {
    const secret = process.env.SECRET;

    jwt.verify(token, secret, (err: any, decoded: any) => {
      if (err) {
        throw new Error(err.name);
      }
      // if everything good, save to request for use in other routes
      const data = decoded.data;
      req.headers["x-data"] = data;
      next();
    });
  } catch (err) {
    return res.status(503).send({
      auth: false,
      status: "NO_OK",
      message: err.message
    });
  }
};
