import { Request, Response, Router } from "express";

import { Petition } from "../../../controllers/http-petition";
import { ExampleController } from "../../../controllers/api/example-controller";

export default class WebRouter {
  private static _instance: Router;

  private constructor() {
    return this;
  }

  private static initRoutes() {
    this._instance.get("/status", (req: Request, res: Response) => {
      res.send("OK").status(200);
    });

    const exampleController = new ExampleController();
    this._instance.get("/example", (req: Request, res: Response) => {
      exampleController.petition = this.getControllerPetition(req, res);
      return exampleController.index();
    });
  }

  private static getControllerPetition(req: Request, res: Response): Petition {
    return {
      request: { ...req, payload: req.body },
      response: { ...res, status: "", headers: null, payload: {} }
    };
  }

  public static get instance() {
    if (!this._instance) {
      this._instance = Router();
      this.initRoutes();
    }
    return this._instance;
  }
}
