import { Sequelize, Optional, Model, DataTypes } from "sequelize";

interface GenericExtr {
  year: number;
  maker: string;
  model: string;
  version: string;
  color: string;
  price: number;
  title: string;
  description: string;
  plate: string;
  mileage: number;
  gearbox: string;
  engine: string;
  fuel: string;
  imageUrl: string;
  code: string;
  url: string;
  region: string;
  city: string;
  sellerType: string;
  sellerName: string;
  publicationCreationDate: Date;
  publicationUpdateDate: Date;
  publicationCreatedAt: Date;
}

interface GenericExtrCreationAttributes extends Optional<GenericExtr, 'publicationCreationDate' | 'publicationUpdateDate'> { }

async function createGenericExtrModel(sequelize: Sequelize, tableName: string) {
  if (!tableName) {
    throw new Error("Table name is required");
  }

  // Obtiene la lista de tablas de la base de datos
  const tables = await sequelize.getQueryInterface().showAllTables();
    
  // Verifica si la tabla está en la lista
  if (!tables.includes(tableName)) {
    throw new Error(`The table "${tableName}" doesn't exists`)
  }

  return sequelize.define<Model<GenericExtr, GenericExtrCreationAttributes>>(tableName, {
    year: DataTypes.INTEGER,
    maker: DataTypes.STRING,
    model: DataTypes.STRING,
    version: DataTypes.STRING,
    color: DataTypes.STRING,
    price: DataTypes.INTEGER,
    title: DataTypes.STRING,
    description: DataTypes.TEXT,
    plate: DataTypes.STRING,
    mileage: DataTypes.INTEGER,
    gearbox: DataTypes.STRING,
    engine: DataTypes.STRING,
    fuel: DataTypes.STRING,
    imageUrl: DataTypes.STRING,
    code: DataTypes.STRING,
    url: DataTypes.STRING,
    region: DataTypes.STRING,
    city: DataTypes.STRING,
    sellerType: DataTypes.STRING,
    sellerName: DataTypes.STRING,
    publicationCreationDate: {
      type: DataTypes.DATE,
      allowNull: false,
      defaultValue: new Date()
    },
    publicationUpdateDate: {
      type: DataTypes.DATE,
      allowNull: true,
      defaultValue: null
    },
    publicationCreatedAt: {
      type: DataTypes.DATE,
      allowNull: false,
      defaultValue: new Date()
    }
  })
}

export default createGenericExtrModel;
